
#ifndef __CONTROLLER_HPP__
#define __CONTROLLER_HPP__

#include "resources.hpp"


// Implementation of the Jt controller.
class Controller : public yarp::os::RateThread
{
protected:
    Matrix *K;

    const Resource &res_gen;
    const Resource &res_mot;
    Resource &res_ctrl;

    double t0;

public:
    Controller(Resource &_res_gen, Resource &_res_ctrl, Resource &_res_mot, const Matrix &_K) :
              RateThread(SAMPLE_TIME), res_gen(_res_gen), res_ctrl(_res_ctrl), res_mot(_res_mot)
    {        
        K=new Matrix(_K);

        // set joints velocities equal to 0.0
        Vector qp(2);
        qp[0]=0.0;
        qp[1]=0.0;

        res_ctrl.set(qp);
    }

    virtual bool threadInit()
    {        
        t0=yarp::os::Time::now();
        return true;
    }

    virtual void run()
    {
        Vector pd=res_gen.get();
        Vector q=res_mot.get();
        Vector pe=phi(q);

        Vector e=pd-pe;

        const double c=cos(q[0]);
        const double s=sin(q[0]);

        // compute the Jacobian matrix.
        // remind that J=[dphi(q)/dq1 dphi(q)/dq2].
        Matrix J(2,2);
        J(0,0)=-q[1]*s; J(0,1)=c;
        J(1,0)= q[1]*c; J(1,1)=s;

        // the core of controller
        Vector qp=J.transpose()* *K *e;
        res_ctrl.set(qp);

        print(pd,pe,qp,q,e);
    }  

    virtual void threadRelease()
    {
        delete K;
    }
    
    // the forward kinematic law
    Vector phi(const Vector &q)
    {
        Vector pe(2);

        pe[0]=q[1]*cos(q[0]);
        pe[1]=q[1]*sin(q[0]);

        return pe;
    }

    void print(const Vector &pd, const Vector &pe, const Vector &qp,
               const Vector &q, const Vector &e)
    {
        const double t=yarp::os::Time::now();

        if (t-t0>=PRINT_INFO_PER)
        {
            Vector qp_meas=qp;
            Vector q_meas =q;

            qp_meas[0]*=(180.0/PI);
            q_meas[0] *=(180.0/PI);

            fprintf(stdout,"pd          [m] = "); pd.print(); 
            fprintf(stdout,"pe          [m] = "); pe.print();
            fprintf(stdout,"qp [deg/s, m/s] = "); qp_meas.print();
            fprintf(stdout,"q      [deg, m] = "); q_meas.print();
            fprintf(stdout,"|e|         [m] = %.3f\n",norm(e));
            fprintf(stdout,"\n");

            t0=t;
        }
    }
};

#endif

