
#ifndef __MATRIX_HPP__
#define __MATRIX_HPP__

#include "vector.hpp"

class Matrix
{
protected:
    Vector **storage;
    unsigned int rows;
    unsigned int cols;

    void disposeHelper()
    {
        for (unsigned int i=0; i<cols; i++)
            delete storage[i];

        delete[] storage;
    }

    void cloneHelper(const Matrix &M)
    {
        rows=M.getRows();
        cols=M.getCols();

        storage=new Vector*[cols];

        for (unsigned int j=0; j<cols; j++)
        {    
            storage[j]=new Vector(rows);

            for (unsigned int i=0; i<rows; i++)
                (*this)(i,j)=M(i,j);
        }
    }

public:
    Matrix(const unsigned int r, const unsigned int c) : rows(r), cols(c)
    {
        storage=new Vector*[cols];

        for (unsigned int i=0; i<cols; i++)
            storage[i]=new Vector(rows);
    }

    Matrix(const Matrix &M)
    {
        cloneHelper(M);
    }

    unsigned int getRows() const
    {
        return rows;
    }


    unsigned int getCols() const
    {
        return cols;
    }

    double &operator()(const unsigned int r, const unsigned int c)
    {
        return (*storage[c])[r];
    }

    double &operator()(const unsigned int r, const unsigned int c) const
    {
        return (*storage[c])[r];
    }

    Vector &operator[](const unsigned int c)
    {
        return *(storage[c]);
    }

    Vector &operator[](const unsigned int c) const
    {
        return *(storage[c]);
    }

    Matrix operator=(const Matrix &M)
    {
        disposeHelper();
        cloneHelper(M);

        return *this;
    }

    Matrix operator+(const Matrix &M) const
    {
        Matrix res(rows,cols);        

        if (M.getRows()==rows && M.getCols()==cols)
            for (unsigned int i=0; i<cols; i++)
                res[i]=(*this)[i]+M[i];

        return res;
    }

    Matrix operator-(const Matrix &M) const
    {
        Matrix res(rows,cols);        

        if (M.getRows()==rows && M.getCols()==cols)
            for (unsigned int i=0; i<cols; i++)
                res[i]=(*this)[i]-M[i];

        return res;
    }

    Matrix transpose() const
    {
        Matrix res(cols,rows);

        for (unsigned int j=0; j<res.getCols(); j++)
            for (unsigned int i=0; i<res.getRows(); i++)
                res(i,j)=(*this)(j,i);

        return res;
    }

    Vector operator*(const Vector &v) const
    {        
        Vector res(rows);

        if (v.getN()==cols)
        {
            Matrix trp=transpose();
                
            for (unsigned int i=0; i<res.getN(); i++)
                res[i]=dot(trp[i],v);
        }

        return res;
    }

    Matrix operator*(const Matrix &M) const
    {        
        Matrix res(rows,M.getCols());

        if (M.getRows()==cols)
            for (unsigned int i=0; i<res.getCols(); i++)
                res[i]=(*this)*M[i];

        return res;
    }

    Matrix operator*(const double c) const
    {        
        Matrix res(rows,cols);

        for (unsigned int i=0; i<res.getCols(); i++)
            res[i]=res[i]*c;

        return res;
    }

    void print() const
    {
        Matrix trp=transpose();

        for (unsigned int i=0; i<trp.getCols(); i++)
            trp[i].print();
    }

    ~Matrix()
    {
        disposeHelper();
    }
};


#endif


