
#ifndef __RESOURCES_HPP__
#define __RESOURCES_HPP__

#include <yarp/os/RateThread.h>
#include <yarp/os/Semaphore.h>
#include <yarp/os/Time.h>

#include <math.h>

#include "vector.hpp"
#include "matrix.hpp"

#define PI              3.14159265359   // [rad]
#define SAMPLE_TIME     20              // [ms]
#define GEN_FREQ        0.1             // [Hz]
#define PRINT_INFO_PER  1.0             // [s]


// This class declares a Vector object which can be shared among
// different threads since the methods get() and set() provide
// safe access to it.
class Resource
{
protected:
    yarp::os::Semaphore *mutex;
    Vector *data;

public:
    Resource(const unsigned int nelem)
    {
        mutex=new yarp::os::Semaphore(1);
        data=new Vector(nelem);
    }

    Vector get() const
    {
        mutex->wait();
        Vector ret=*data;
        mutex->post();

        return ret;
    }

    void set(const Vector &v)
    {
        mutex->wait();
        *data=v;
        mutex->post();
    }

    ~Resource()
    {
        delete mutex;
        delete data;
    }
};

#endif


