
#ifndef __VECTOR_HPP__
#define __VECTOR_HPP__

#include <stdlib.h>
#include <stdio.h>
#include <math.h>

class Vector
{
protected:
    double *storage;
    unsigned int N;

    void disposeHelper()
    {
        delete[] storage;
    }

    void cloneHelper(const Vector &v)
    {
        N=v.getN();

        if (N)
            storage=new double[N];

        for (unsigned int i=0; i<N; i++)
            storage[i]=v[i];
    }

public:
    Vector(const unsigned int nelem)
    {
        N=nelem;

        if (N)
            storage=new double[N];
    }

    Vector(const Vector &v)
    {
        cloneHelper(v);
    }

    unsigned int getN() const
    {
        return N;
    }

    double &operator[](const unsigned int i)
    {
        return storage[i];
    }

    double &operator[](const unsigned int i) const
    {
        return storage[i];
    }

    Vector operator=(const Vector &v)
    {
        disposeHelper();
        cloneHelper(v);

        return *this;
    }

    Vector operator+(const Vector &v) const
    {
        Vector res=*this;

        if (v.getN()==N)
            for (unsigned int i=0; i<N; i++)
                res[i]+=v[i];

        return res;
    }

    Vector operator-(const Vector &v) const
    {
        Vector res=*this;

        if (v.getN()==N)
            for (unsigned int i=0; i<N; i++)
                res[i]-=v[i];

        return res;
    }

    Vector operator*(const Vector &v) const
    {
        Vector res=*this;

        if (v.getN()==N)
            for (unsigned int i=0; i<N; i++)
                res[i]*=v[i];

        return res;
    }

    Vector operator*(const double c) const
    {
        Vector res=*this;

        for (unsigned int i=0; i<N; i++)
            res[i]*=c;

        return res;
    }

    void print() const
    {
        for (unsigned int i=0; i<N; i++)
            fprintf(stdout,"%.3f ",storage[i]);

        fprintf(stdout,"\n");
    }

    ~Vector()
    {
        disposeHelper();
    }
};


double dot(const Vector &v1, const Vector &v2)
{
    double res=0.0;

    if (v1.getN()==v2.getN())
        for (unsigned int i=0; i<v1.getN(); i++)
            res+=v1[i]*v2[i];

    return res;
}


double norm(const Vector &v)
{
    return sqrt(dot(v,v));
}


#endif


