
// Model a control system for a serial kinematic chain composed
// of a rotational joint and prismatic joint [theta; r]
// with theta in [0,2*PI] and r in [0,+inf], so as it would represent
// any 2-d point in the polar coordinates.

// disable the warning on scanf()
#pragma warning(disable:4996)

#include <stdlib.h>

#include "resources.hpp"
#include "generator.hpp"
#include "controller.hpp"
#include "motor.hpp"


int main()
{
    yarp::os::Time::turboBoost();

    // define the three shared resources
    Resource q(2);
    Resource qp(2);
    Resource pd(2);

    // define the gain matrix
    // feel free to play with these values
    // in order to change convergence speed
    Matrix K(2,2);
    K(0,0)=10.0; K(0,1)=0.0;
    K(1,0)=0.0;  K(1,1)=10.0;

    // initial motor position
    Vector q0(2);
    q0[0]=PI/2;
    q0[1]=2.0;

    // define the motor, the controller and the generator
    Motor            mot(qp,q,q0);
    Controller       ctrl(pd,qp,q,K);
    EllipseGenerator gen(pd,1.0,1.0);

    // start the threads
    mot.start();
    ctrl.start();
    gen.start();

    // wait until a key is pressed
    char ch;
    scanf("%c",&ch);

    // stop the threads
    gen.stop();
    ctrl.stop();
    mot.stop();

    return 0;
}


